﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Gallio.Framework;
using MbUnit.Framework;
using PostSharp.Laos;


namespace SkillsMatter.PostSharp.Aspects
{
	[Serializable]
	public class RangeValidator : OnMethodBoundaryAspect
	{
		public int LowerBoundary { get; set; }
		public int UpperBoundary { get; set; }

		public RangeValidator(int lower, int upper)
		{
			LowerBoundary = lower;
			UpperBoundary = upper;
		}

		public override void OnEntry(MethodExecutionEventArgs eventArgs)
		{
			if (eventArgs.Method.Name.StartsWith("get_"))
				return;

			var args = eventArgs.GetReadOnlyArgumentArray();
			if (args.Count() == 0)
				throw new ValidationException("No argument passed to set");

			int value = (int)args[0];
			if (value > UpperBoundary || value < LowerBoundary)
				throw new ValidationException("Value must be between " + LowerBoundary + " and " 
					+ UpperBoundary);


			base.OnEntry(eventArgs);
		}

	}
}
